@@ -21,17 +21,19 @@ enum {
 	R_SYS_BITS_64 = 8,
 };
 
-typedef struct _os_info {
-	char name[32];
-	int major;
-	int minor;
-	int patch;
-	int compilation;
-} os_info;
+typedef struct {
+	char *sysname;
+	char *nodename;
+	char *release;
+	char *version;
+	char *machine;
+} RSysInfo;
+
+R_API RSysInfo *r_sys_info(void);
+R_API void r_sys_info_free(RSysInfo *si);
 
 R_API char **r_sys_get_environ(void);
 R_API void r_sys_set_environ(char **e);
-R_API os_info *r_sys_get_osinfo(void);
 R_API ut64 r_sys_now(void);
 R_API const char *r_time_to_string (ut64 ts);
 R_API int r_sys_fork(void);
@@ -41,6 +43,7 @@ R_API bool r_is_heap (void *p);
 R_API bool r_sys_stop(void);
 R_API char *r_sys_pid_to_path(int pid);
 R_API int r_sys_run(const ut8 *buf, int len);
+R_API int r_sys_run_rop(const ut8 *buf, int len);
 R_API int r_sys_getpid(void);
 R_API int r_sys_crash_handler(const char *cmd);
 R_API const char *r_sys_arch_str(int arch);
@@ -59,12 +62,14 @@ R_API bool r_sys_mkdirp(const char *dir);
 R_API int r_sys_sleep(int secs);
 R_API int r_sys_usleep(int usecs);
 R_API char *r_sys_getenv(const char *key);
+R_API bool r_sys_getenv_asbool(const char *key);
 R_API int r_sys_setenv(const char *key, const char *value);
 R_API int r_sys_clearenv(void);
 R_API char *r_sys_whoami(char *buf);
 R_API char *r_sys_getdir(void);
 R_API int r_sys_chdir(const char *s);
 R_API bool r_sys_aslr(int val);
+R_API int r_sys_thp_mode(void);
 R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output, int *len, char **sterr);
 #if __WINDOWS__
 #if UNICODE
@@ -82,7 +87,6 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, char **output,
 #define r_sys_conv_win_to_utf8(buf) r_acp_to_utf8 (buf)
 #define r_sys_conv_win_to_utf8_l(buf, len) r_acp_to_utf8_l (buf, len)
 #endif
-R_API os_info *r_sys_get_winver(void);
 R_API char *r_sys_get_src_dir_w32(void);
 R_API bool r_sys_cmd_str_full_w32(const char *cmd, const char *input, char **output, int *outlen, char **sterr);
 R_API bool r_sys_create_child_proc_w32(const char *cmdline, HANDLE in, HANDLE out, HANDLE err);
@@ -98,7 +102,6 @@ R_API void r_sys_backtrace(void);
 R_API bool r_sys_tts(const char *txt, bool bg);
 
 #if __WINDOWS__
-#include <intrin.h>
 #  define r_sys_breakpoint() { __debugbreak  (); }
 #else
 #if __GNUC__
